@@ -44,10 +44,6 @@ PLUGIN_SET_TRANSLATABLE_INFO(
 	_(	"This plugin adds features to facilitate navigation between source files."), 
 	CODE_NAVIGATION_VERSION, "Lionel Fuentes, Federico Reghenzani")
 
-/* Declare "GeanyKeyGroupInfo plugin_key_group_info[1]" and "GeanyKeyGroup *plugin_key_group",
- * for Geany to find the keybindings */
-PLUGIN_KEY_GROUP(code_navigation, NB_KEY_IDS)
-
 
 /********************* Data types for the feature *********************/
 /* Column for the configuration widget */
@@ -64,6 +60,8 @@ typedef enum
 GeanyPlugin		*geany_plugin;
 GeanyData		*geany_data;
 
+GeanyKeyGroup *plugin_key_group;
+
 static GtkListStore *list_store;	/* for settings dialog */
 
 
@@ -100,6 +98,8 @@ void plugin_init(GeanyData *data)
 {
 	log_func();
 
+	plugin_key_group = plugin_set_key_group(geany_plugin, "code_navigation", NB_KEY_IDS, NULL);
+
 	/* Load configuration */
 	load_configuration();
 	/* Initialize the features */
