@@ -112,7 +112,7 @@ PluginCallback plugin_callbacks[] = {
 
 void plugin_help (void)
 {
-	utils_open_browser (DOCDIR "/" PLUGIN "/README");
+	utils_open_browser("http://plugins.geany.org/geanyctags.html");
 }
 
 static void spawn_cmd(const gchar *cmd, const gchar *dir)
