@@ -39,19 +39,39 @@
 
 #ifdef USE_GTKSPELL
 #include <gtkspell/gtkspell.h>
+/* forward compatibility with GtkSpell3 */
+#if GTK_CHECK_VERSION(3, 0, 0)
+#define GtkSpell GtkSpellChecker
+#define gtkspell_set_language gtk_spell_checker_set_language
+static GtkSpell *gtkspell_new_attach(GtkTextView *view, const gchar *lang, GError **error)
+{
+	GtkSpellChecker *speller = gtk_spell_checker_new();
+
+	if (! lang || gtk_spell_checker_set_language(speller, lang, error))
+		gtk_spell_checker_attach(speller, view);
+	else
+	{
+		g_object_unref(g_object_ref_sink(speller));
+		speller = NULL;
+	}
+
+	return speller;
+}
+#endif
 #endif
 
 GeanyData *geany_data;
-GeanyFunctions *geany_functions;
 GeanyPlugin		*geany_plugin;
 
 
-PLUGIN_VERSION_CHECK(188)
+PLUGIN_VERSION_CHECK(224)
 PLUGIN_SET_TRANSLATABLE_INFO(
 	LOCALEDIR,
 	GETTEXT_PACKAGE,
 	_("GeanyVC"),
-	_("Interface to different Version Control systems."),
+	_("Interface to different Version Control systems."
+	  "\nThis plugins is currently not developed actively. "
+	  "Would you like to help by contributing to this plugin?"),
 	VERSION,
 	"Yura Siamashka <yurand2@gmail.com>,\nFrank Lanitz <frank@frank.uvena.de>")
 
@@ -62,6 +82,8 @@ static gboolean set_maximize_commit_dialog;
 static gboolean set_external_diff;
 static gboolean set_editor_menu_entries;
 static gboolean set_menubar_entry;
+static gint commit_dialog_width = 0;
+static gint commit_dialog_height = 0;
 
 static gchar *config_file;
 
@@ -899,7 +921,7 @@ command_with_question_activated(gchar ** text, gint cmd, const gchar * question,
 		if (flags & (FLAG_DIR | FLAG_BASEDIR))
 			execute_command(vc, text, NULL, dir, cmd, NULL, NULL);
 		if (flags & FLAG_RELOAD)
-			document_reload_file(doc, NULL);
+			document_reload_force(doc, NULL);
 	}
 	g_free(dir);
 	return (result == GTK_RESPONSE_YES);
@@ -957,7 +979,7 @@ vcupdate_activated(G_GNUC_UNUSED GtkMenuItem * menuitem, G_GNUC_UNUSED gpointer
 	if (command_with_question_activated(&text, VC_COMMAND_UPDATE,
 					    _("Do you really want to update?"), FLAG_BASEDIR))
 	{
-		document_reload_file(doc, NULL);
+		document_reload_force(doc, NULL);
 
 		if (!EMPTY(text))
 			show_output(text, "*VC-UPDATE*", NULL, NULL, 0);
@@ -1578,6 +1600,8 @@ vccommit_activated(G_GNUC_UNUSED GtkMenuItem * menuitem, G_GNUC_UNUSED gpointer
 	else
 	{
 		gtk_widget_set_size_request(commit, 700, 500);
+		gtk_window_set_default_size(GTK_WINDOW(commit),
+			commit_dialog_width, commit_dialog_height);
 	}
 
 	gtk_widget_show_now(commit);
@@ -1586,27 +1610,14 @@ vccommit_activated(G_GNUC_UNUSED GtkMenuItem * menuitem, G_GNUC_UNUSED gpointer
 	gtk_paned_set_position(GTK_PANED(vpaned2), height * 50 / 100);
 
 #ifdef USE_GTKSPELL
-	speller = gtkspell_new_attach(GTK_TEXT_VIEW(messageView), NULL, &spellcheck_error);
-	if (speller == NULL)
+	speller = gtkspell_new_attach(GTK_TEXT_VIEW(messageView), EMPTY(lang) ? NULL : lang, &spellcheck_error);
+	if (speller == NULL && spellcheck_error != NULL)
 	{
-		ui_set_statusbar(FALSE, _("Error initializing spell checking: %s"),
+		ui_set_statusbar(TRUE, _("Error initializing GeanyVC spell checking: %s. Check your configuration."),
 				 spellcheck_error->message);
 		g_error_free(spellcheck_error);
 		spellcheck_error = NULL;
 	}
-	else if (!EMPTY(lang))
-	{
-		gtkspell_set_language(speller, lang, &spellcheck_error);
-		if (spellcheck_error != NULL)
-		{
-			ui_set_statusbar(TRUE,
-					 _
-					 ("Error while setting up language for spellchecking. Please check configuration. Error message was: %s"),
-					 spellcheck_error->message);
-			g_error_free(spellcheck_error);
-			spellcheck_error = NULL;
-		}
-	}
 #endif
 
 	/* put the input focus to the commit message text view */
@@ -1628,6 +1639,9 @@ vccommit_activated(G_GNUC_UNUSED GtkMenuItem * menuitem, G_GNUC_UNUSED gpointer
 		}
 		g_free(message);
 	}
+	/* remember commit dialog widget size */
+	gtk_window_get_size(GTK_WINDOW(commit),
+		&commit_dialog_width, &commit_dialog_height);
 
 	gtk_widget_destroy(commit);
 	free_commit_list(lst);
@@ -1778,16 +1792,66 @@ static struct
 }
 widgets;
 
+static void
+save_config(void)
+{
+	GKeyFile *config = g_key_file_new();
+	gchar *config_dir = g_path_get_dirname(config_file);
+
+	g_key_file_load_from_file(config, config_file, G_KEY_FILE_NONE, NULL);
+
+	g_key_file_set_boolean(config, "VC", "set_changed_flag", set_changed_flag);
+	g_key_file_set_boolean(config, "VC", "set_add_confirmation", set_add_confirmation);
+	g_key_file_set_boolean(config, "VC", "set_external_diff", set_external_diff);
+	g_key_file_set_boolean(config, "VC", "set_maximize_commit_dialog",
+			       set_maximize_commit_dialog);
+	g_key_file_set_boolean(config, "VC", "set_editor_menu_entries", set_editor_menu_entries);
+	g_key_file_set_boolean(config, "VC", "attach_to_menubar", set_menubar_entry);
+
+	g_key_file_set_boolean(config, "VC", "enable_cvs", enable_cvs);
+	g_key_file_set_boolean(config, "VC", "enable_git", enable_git);
+	g_key_file_set_boolean(config, "VC", "enable_svn", enable_svn);
+	g_key_file_set_boolean(config, "VC", "enable_svk", enable_svk);
+	g_key_file_set_boolean(config, "VC", "enable_bzr", enable_bzr);
+	g_key_file_set_boolean(config, "VC", "enable_hg", enable_hg);
+
+#ifdef USE_GTKSPELL
+	g_key_file_set_string(config, "VC", "spellchecking_language", lang);
+#endif
+
+	if (commit_dialog_width > 0 && commit_dialog_height > 0)
+	{
+		g_key_file_set_integer(config, "CommitDialog",
+			"commit_dialog_width", commit_dialog_width);
+		g_key_file_set_integer(config, "CommitDialog",
+			"commit_dialog_height", commit_dialog_height);
+	}
+
+	if (!g_file_test(config_dir, G_FILE_TEST_IS_DIR)
+	    && utils_mkdir(config_dir, TRUE) != 0)
+	{
+		dialogs_show_msgbox(GTK_MESSAGE_ERROR,
+				    _
+				    ("Plugin configuration directory could not be created."));
+	}
+	else
+	{
+		/* write config to file */
+		gchar *data = g_key_file_to_data(config, NULL, NULL);
+		utils_write_file(config_file, data);
+		g_free(data);
+	}
+
+	g_free(config_dir);
+	g_key_file_free(config);
+}
+
 static void
 on_configure_response(G_GNUC_UNUSED GtkDialog * dialog, gint response,
 		      G_GNUC_UNUSED gpointer user_data)
 {
 	if (response == GTK_RESPONSE_OK || response == GTK_RESPONSE_APPLY)
 	{
-		GKeyFile *config = g_key_file_new();
-		gchar *data;
-		gchar *config_dir = g_path_get_dirname(config_file);
-
 		set_changed_flag =
 			gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widgets.cb_changed_flag));
 		set_add_confirmation =
@@ -1815,51 +1879,13 @@ on_configure_response(G_GNUC_UNUSED GtkDialog * dialog, gint response,
 		lang = g_strdup(gtk_entry_get_text(GTK_ENTRY(widgets.spellcheck_lang_textbox)));
 #endif
 
-		g_key_file_load_from_file(config, config_file, G_KEY_FILE_NONE, NULL);
-
-		g_key_file_set_boolean(config, "VC", "set_changed_flag", set_changed_flag);
-		g_key_file_set_boolean(config, "VC", "set_add_confirmation", set_add_confirmation);
-		g_key_file_set_boolean(config, "VC", "set_external_diff", set_external_diff);
-		g_key_file_set_boolean(config, "VC", "set_maximize_commit_dialog",
-				       set_maximize_commit_dialog);
-		g_key_file_set_boolean(config, "VC", "set_editor_menu_entries", set_editor_menu_entries);
-		g_key_file_set_boolean(config, "VC", "attach_to_menubar", set_menubar_entry);
-
-		g_key_file_set_boolean(config, "VC", "enable_cvs", enable_cvs);
-		g_key_file_set_boolean(config, "VC", "enable_git", enable_git);
-		g_key_file_set_boolean(config, "VC", "enable_svn", enable_svn);
-		g_key_file_set_boolean(config, "VC", "enable_svk", enable_svk);
-		g_key_file_set_boolean(config, "VC", "enable_bzr", enable_bzr);
-		g_key_file_set_boolean(config, "VC", "enable_hg", enable_hg);
-
-#ifdef USE_GTKSPELL
-		g_key_file_set_string(config, "VC", "spellchecking_language", lang);
-#endif
-
-		if (!g_file_test(config_dir, G_FILE_TEST_IS_DIR)
-		    && utils_mkdir(config_dir, TRUE) != 0)
-		{
-			dialogs_show_msgbox(GTK_MESSAGE_ERROR,
-					    _
-					    ("Plugin configuration directory could not be created."));
-		}
-		else
-		{
-			/* write config to file */
-			data = g_key_file_to_data(config, NULL, NULL);
-			utils_write_file(config_file, data);
-			g_free(data);
-		}
+		save_config();
 
 		if (set_editor_menu_entries == FALSE)
 			remove_menuitems_from_editor_menu();
 		else
 			add_menuitems_to_editor_menu();
 
-
-		g_free(config_dir);
-		g_key_file_free(config);
-
 		registrate();
 	}
 }
@@ -1878,7 +1904,7 @@ plugin_configure(GtkDialog * dialog)
 	widgets.cb_changed_flag =
 		gtk_check_button_new_with_label(_
 						("Set Changed-flag for document tabs created by the plugin"));
-	ui_widget_set_tooltip_text(widgets.cb_changed_flag,
+	gtk_widget_set_tooltip_text(widgets.cb_changed_flag,
 			     _
 			     ("If this option is activated, every new by the VC-plugin created document tab "
 			      "will be marked as changed. Even this option is useful in some cases, it could cause "
@@ -1889,7 +1915,7 @@ plugin_configure(GtkDialog * dialog)
 
 	widgets.cb_confirm_add =
 		gtk_check_button_new_with_label(_("Confirm adding new files to a VCS"));
-	ui_widget_set_tooltip_text(widgets.cb_confirm_add,
+	gtk_widget_set_tooltip_text(widgets.cb_confirm_add,
 			     _
 			     ("Shows a confirmation dialog on adding a new (created) file to VCS."));
 	gtk_button_set_focus_on_click(GTK_BUTTON(widgets.cb_confirm_add), FALSE);
@@ -1898,14 +1924,14 @@ plugin_configure(GtkDialog * dialog)
 	gtk_box_pack_start(GTK_BOX(vbox), widgets.cb_confirm_add, TRUE, FALSE, 2);
 
 	widgets.cb_max_commit = gtk_check_button_new_with_label(_("Maximize commit dialog"));
-	ui_widget_set_tooltip_text(widgets.cb_max_commit, _("Show commit dialog maximize."));
+	gtk_widget_set_tooltip_text(widgets.cb_max_commit, _("Show commit dialog maximize."));
 	gtk_button_set_focus_on_click(GTK_BUTTON(widgets.cb_max_commit), FALSE);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets.cb_max_commit),
 				     set_maximize_commit_dialog);
 	gtk_box_pack_start(GTK_BOX(vbox), widgets.cb_max_commit, TRUE, FALSE, 2);
 
 	widgets.cb_external_diff = gtk_check_button_new_with_label(_("Use external diff viewer"));
-	ui_widget_set_tooltip_text(widgets.cb_external_diff,
+	gtk_widget_set_tooltip_text(widgets.cb_external_diff,
 			     _("Use external diff viewer for file diff."));
 	gtk_button_set_focus_on_click(GTK_BUTTON(widgets.cb_external_diff), FALSE);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets.cb_external_diff),
@@ -1913,14 +1939,14 @@ plugin_configure(GtkDialog * dialog)
 	gtk_box_pack_start(GTK_BOX(vbox), widgets.cb_external_diff, TRUE, FALSE, 2);
 
 	widgets.cb_editor_menu_entries = gtk_check_button_new_with_label(_("Show VC entries at editor menu"));
-	ui_widget_set_tooltip_text(widgets.cb_editor_menu_entries,
+	gtk_widget_set_tooltip_text(widgets.cb_editor_menu_entries,
 			     _("Show entries for VC functions inside editor menu"));
 	gtk_button_set_focus_on_click(GTK_BUTTON(widgets.cb_editor_menu_entries), FALSE);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets.cb_editor_menu_entries), set_editor_menu_entries);
 	gtk_box_pack_start(GTK_BOX(vbox), widgets.cb_editor_menu_entries, TRUE, FALSE, 2);
 
 	widgets.cb_attach_to_menubar = gtk_check_button_new_with_label(_("Attach menu to menubar"));
-	ui_widget_set_tooltip_text(widgets.cb_editor_menu_entries,
+	gtk_widget_set_tooltip_text(widgets.cb_editor_menu_entries,
 			     _("Whether menu for this plugin are getting placed either "
 			       "inside tools menu or directly inside Geany's menubar."
 			       "Will take in account after next start of GeanyVC"));
@@ -2023,6 +2049,11 @@ load_config(void)
 	}
 #endif
 
+	commit_dialog_width = utils_get_setting_integer(config, "CommitDialog",
+		"commit_dialog_width", 700);
+	commit_dialog_height = utils_get_setting_integer(config, "CommitDialog",
+		"commit_dialog_height", 500);
+
 	g_key_file_free(config);
 }
 
@@ -2059,7 +2090,7 @@ do_current_file_menu(GtkWidget ** parent_menu, gboolean editor_menu)
 	/* Diff of current file */
 	menu_vc_diff_file = gtk_menu_item_new_with_mnemonic(_("_Diff"));
 	gtk_container_add(GTK_CONTAINER(cur_file_menu), menu_vc_diff_file);
-	ui_widget_set_tooltip_text(menu_vc_diff_file,
+	gtk_widget_set_tooltip_text(menu_vc_diff_file,
 			     _("Make a diff from the current active file"));
 
 	g_signal_connect(menu_vc_diff_file, "activate", G_CALLBACK(vcdiff_file_activated), NULL);
@@ -2067,7 +2098,7 @@ do_current_file_menu(GtkWidget ** parent_menu, gboolean editor_menu)
 	/* Revert current file */
 	menu_vc_revert_file = gtk_menu_item_new_with_mnemonic(_("_Revert"));
 	gtk_container_add(GTK_CONTAINER(cur_file_menu), menu_vc_revert_file);
-	ui_widget_set_tooltip_text(menu_vc_revert_file,
+	gtk_widget_set_tooltip_text(menu_vc_revert_file,
 			     _("Restore pristine working copy file (undo local edits)."));
 
 	g_signal_connect(menu_vc_revert_file, "activate", G_CALLBACK(vcrevert_activated), NULL);
@@ -2079,7 +2110,7 @@ do_current_file_menu(GtkWidget ** parent_menu, gboolean editor_menu)
 	/* Blame for current file */
 	menu_vc_blame = gtk_menu_item_new_with_mnemonic(_("_Blame"));
 	gtk_container_add(GTK_CONTAINER(cur_file_menu), menu_vc_blame);
-	ui_widget_set_tooltip_text(menu_vc_blame,
+	gtk_widget_set_tooltip_text(menu_vc_blame,
 			     _("Shows the changes made at one file per revision and author."));
 
 	g_signal_connect(menu_vc_blame, "activate", G_CALLBACK(vcblame_activated), NULL);
@@ -2089,7 +2120,7 @@ do_current_file_menu(GtkWidget ** parent_menu, gboolean editor_menu)
 	/* History/log of current file */
 	menu_vc_log_file = gtk_menu_item_new_with_mnemonic(_("_History (log)"));
 	gtk_container_add(GTK_CONTAINER(cur_file_menu), menu_vc_log_file);
-	ui_widget_set_tooltip_text(menu_vc_log_file,
+	gtk_widget_set_tooltip_text(menu_vc_log_file,
 			     _("Shows the log of the current file"));
 
 	g_signal_connect(menu_vc_log_file, "activate", G_CALLBACK(vclog_file_activated), NULL);
@@ -2097,7 +2128,7 @@ do_current_file_menu(GtkWidget ** parent_menu, gboolean editor_menu)
 	/* base version of the current file */
 	menu_vc_show_file = gtk_menu_item_new_with_mnemonic(_("_Original"));
 	gtk_container_add(GTK_CONTAINER(cur_file_menu), menu_vc_show_file);
-	ui_widget_set_tooltip_text(menu_vc_show_file,
+	gtk_widget_set_tooltip_text(menu_vc_show_file,
 			     _("Shows the original of the current file"));
 
 	g_signal_connect(menu_vc_show_file, "activate", G_CALLBACK(vcshow_file_activated), NULL);
@@ -2108,7 +2139,7 @@ do_current_file_menu(GtkWidget ** parent_menu, gboolean editor_menu)
 	/* add current file */
 	menu_vc_add_file = gtk_menu_item_new_with_mnemonic(_("_Add to Version Control"));
 	gtk_container_add(GTK_CONTAINER(cur_file_menu), menu_vc_add_file);
-	ui_widget_set_tooltip_text(menu_vc_add_file, _("Add file to repository."));
+	gtk_widget_set_tooltip_text(menu_vc_add_file, _("Add file to repository."));
 
 	g_signal_connect(menu_vc_add_file, "activate",
 			 G_CALLBACK(vcadd_activated), NULL);
@@ -2116,7 +2147,7 @@ do_current_file_menu(GtkWidget ** parent_menu, gboolean editor_menu)
 	/* remove current file */
 	menu_vc_remove_file = gtk_menu_item_new_with_mnemonic(_("_Remove from Version Control"));
 	gtk_container_add(GTK_CONTAINER(cur_file_menu), menu_vc_remove_file);
-	ui_widget_set_tooltip_text(menu_vc_remove_file, _("Remove file from repository."));
+	gtk_widget_set_tooltip_text(menu_vc_remove_file, _("Remove file from repository."));
 
 	g_signal_connect(menu_vc_remove_file, "activate", G_CALLBACK(vcremove_activated), NULL);
 
@@ -2136,7 +2167,7 @@ do_current_dir_menu(GtkWidget ** parent_menu)
 	/* Diff of the current dir */
 	menu_vc_diff_dir = gtk_menu_item_new_with_mnemonic(_("_Diff"));
 	gtk_container_add(GTK_CONTAINER(cur_dir_menu), menu_vc_diff_dir);
-	ui_widget_set_tooltip_text(menu_vc_diff_dir,
+	gtk_widget_set_tooltip_text(menu_vc_diff_dir,
 			     _("Make a diff from the directory of the current active file"));
 
 	g_signal_connect(menu_vc_diff_dir, "activate",
@@ -2145,7 +2176,7 @@ do_current_dir_menu(GtkWidget ** parent_menu)
 	/* Revert current dir */
 	menu_vc_revert_dir = gtk_menu_item_new_with_mnemonic(_("_Revert"));
 	gtk_container_add(GTK_CONTAINER(cur_dir_menu), menu_vc_revert_dir);
-	ui_widget_set_tooltip_text(menu_vc_revert_dir,
+	gtk_widget_set_tooltip_text(menu_vc_revert_dir,
 			     _("Restore original files in the current folder (undo local edits)."));
 
 	g_signal_connect(menu_vc_revert_dir, "activate",
@@ -2155,7 +2186,7 @@ do_current_dir_menu(GtkWidget ** parent_menu)
 	/* History/log of the current dir */
 	menu_vc_log_dir = gtk_menu_item_new_with_mnemonic(_("_History (log)"));
 	gtk_container_add(GTK_CONTAINER(cur_dir_menu), menu_vc_log_dir);
-	ui_widget_set_tooltip_text(menu_vc_log_dir,
+	gtk_widget_set_tooltip_text(menu_vc_log_dir,
 			     _("Shows the log of the current directory"));
 
 
@@ -2176,7 +2207,7 @@ do_basedir_menu(GtkWidget ** parent_menu)
 	/* Complete diff of base directory */
 	menu_vc_diff_basedir = gtk_menu_item_new_with_mnemonic(_("_Diff"));
 	gtk_container_add(GTK_CONTAINER(basedir_menu), menu_vc_diff_basedir);
-	ui_widget_set_tooltip_text(menu_vc_diff_basedir, _("Make a diff from the top VC directory"));
+	gtk_widget_set_tooltip_text(menu_vc_diff_basedir, _("Make a diff from the top VC directory"));
 
 	g_signal_connect(menu_vc_diff_basedir, "activate",
 			 G_CALLBACK(vcdiff_dir_activated), GINT_TO_POINTER(FLAG_BASEDIR));
@@ -2184,7 +2215,7 @@ do_basedir_menu(GtkWidget ** parent_menu)
 	/* Revert everything */
 	menu_vc_revert_basedir = gtk_menu_item_new_with_mnemonic(_("_Revert"));
 	gtk_container_add(GTK_CONTAINER(basedir_menu), menu_vc_revert_basedir);
-	ui_widget_set_tooltip_text(menu_vc_revert_basedir, _("Revert any local edits."));
+	gtk_widget_set_tooltip_text(menu_vc_revert_basedir, _("Revert any local edits."));
 
 	g_signal_connect(menu_vc_revert_basedir, "activate",
 			 G_CALLBACK(vcrevert_dir_activated), GINT_TO_POINTER(FLAG_BASEDIR));
@@ -2196,7 +2227,7 @@ do_basedir_menu(GtkWidget ** parent_menu)
 	/* Complete History/Log of base directory */
 	menu_vc_log_basedir = gtk_menu_item_new_with_mnemonic(_("_History (log)"));
 	gtk_container_add(GTK_CONTAINER(basedir_menu), menu_vc_log_basedir);
-	ui_widget_set_tooltip_text(menu_vc_log_basedir,
+	gtk_widget_set_tooltip_text(menu_vc_log_basedir,
 			     _("Shows the log of the top VC directory"));
 
 	g_signal_connect(menu_vc_log_basedir, "activate",
@@ -2300,12 +2331,15 @@ plugin_init(G_GNUC_UNUSED GeanyData * data)
 	if (set_menubar_entry == TRUE)
 	{
 		GtkMenuShell *menubar;
+		GList *menubar_children;
 
 		menubar = GTK_MENU_SHELL(
 				ui_lookup_widget(geany->main_widgets->window, "menubar1"));
 
 		menu_vc = gtk_menu_item_new_with_mnemonic(_("_VC"));
-		gtk_menu_shell_append(menubar, menu_vc);
+		menubar_children = gtk_container_get_children(GTK_CONTAINER(menubar));
+		gtk_menu_shell_insert(menubar, menu_vc, g_list_length(menubar_children) - 1);
+		g_list_free(menubar_children);
 	}
 	else
 	{
@@ -2333,21 +2367,21 @@ plugin_init(G_GNUC_UNUSED GeanyData * data)
 	/* Status of basedir */
 	menu_vc_status = gtk_menu_item_new_with_mnemonic(_("_Status"));
 	gtk_container_add(GTK_CONTAINER(menu_vc_menu), menu_vc_status);
-	ui_widget_set_tooltip_text(menu_vc_status, _("Show status."));
+	gtk_widget_set_tooltip_text(menu_vc_status, _("Show status."));
 
 	g_signal_connect(menu_vc_status, "activate", G_CALLBACK(vcstatus_activated), NULL);
 
 	/* complete update */
 	menu_vc_update = gtk_menu_item_new_with_mnemonic(_("_Update"));
 	gtk_container_add(GTK_CONTAINER(menu_vc_menu), menu_vc_update);
-	ui_widget_set_tooltip_text(menu_vc_update, _("Update from remote repository."));
+	gtk_widget_set_tooltip_text(menu_vc_update, _("Update from remote repository."));
 
 	g_signal_connect(menu_vc_update, "activate", G_CALLBACK(vcupdate_activated), NULL);
 
 	/* Commit all changes */
 	menu_vc_commit = gtk_menu_item_new_with_mnemonic(_("_Commit"));
 	gtk_container_add(GTK_CONTAINER(menu_vc_menu), menu_vc_commit);
-	ui_widget_set_tooltip_text(menu_vc_commit, _("Commit changes."));
+	gtk_widget_set_tooltip_text(menu_vc_commit, _("Commit changes."));
 
 	g_signal_connect(menu_vc_commit, "activate", G_CALLBACK(vccommit_activated), NULL);
 
@@ -2368,6 +2402,7 @@ plugin_init(G_GNUC_UNUSED GeanyData * data)
 void
 plugin_cleanup(void)
 {
+	save_config();
 	external_diff_viewer_deinit();
 	remove_menuitems_from_editor_menu();
 	gtk_widget_destroy(menu_entry);
