@@ -17,7 +17,9 @@
  *  along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#define _XOPEN_SOURCE 500 /* for openpty, grantpt and unlockpt */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <errno.h>
 #include <stdlib.h>
@@ -30,16 +32,17 @@
 #define DS_COPY (DS_BASICS | DS_EXTRA_1)
 
 #ifdef G_OS_UNIX
-#if (defined(__unix__) || defined(unix)) && !defined(USG)
-#include <sys/param.h>
-#endif
-#ifdef BSD
-#include <termios.h>
+#include <vte/vte.h>
+/* instead of detecting N kinds of *nix */
+#if defined(HAVE_UTIL_H)
+#include <util.h>
+#elif defined(HAVE_LIBUTIL_H)
 #include <libutil.h>
-#else
+#elif defined(HAVE_PTY_H)
 #include <pty.h>
 #endif
-#include <vte/vte.h>
+int grantpt(int fd);
+int unlockpt(int fd);
 
 static GtkWidget *program_window;
 static VteTerminal *program_terminal;
