@@ -17,6 +17,10 @@
  *  along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include <ctype.h>
 #include <errno.h>
 #include <limits.h>
@@ -727,7 +731,7 @@ void utils_init(void)
 
 void utils_finalize(void)
 {
-	guint i;
+	guint i = 0;
 	DebugState state = debug_state();
 
 	foreach_document(i)
