@@ -41,7 +41,7 @@ enum
 	DATA_ID_PRJ_BOOKMARK,
 	DATA_ID_DIRECTORY,
 	DATA_ID_NO_DIRS,
-	DATA_ID_FOLDER,
+	DATA_ID_SUB_DIRECTORY,
 	DATA_ID_FILE,
 };
 
@@ -90,13 +90,13 @@ static void sidebar_remove_children(GtkTreeIter *parent)
 }
 
 
-/* Create a branch for a folder */
+/* Create a branch for a sub-directory */
 static void sidebar_create_branch(gint level, const gchar *abs_base_dir, GSList *leaf_list, GtkTreeIter *parent)
 {
 	GSList *dir_list = NULL;
 	GSList *file_list = NULL;
 	GSList *elem = NULL;
-	gchar **path_arr;
+	gchar **path_arr, *part, *full, *dirpath;
 
 	foreach_slist (elem, leaf_list)
 	{
@@ -105,6 +105,10 @@ static void sidebar_create_branch(gint level, const gchar *abs_base_dir, GSList
 			continue;
 		}
 		path_arr = elem->data;
+		if (path_arr[level] == NULL)
+		{
+			continue;
+		}
 
 		if (path_arr[level+1] != NULL)
 		{
@@ -112,14 +116,24 @@ static void sidebar_create_branch(gint level, const gchar *abs_base_dir, GSList
 		}
 		else
 		{
-			file_list = g_slist_prepend(file_list, path_arr);
+			// Extra check for empty directories
+			part = g_build_filenamev(path_arr);
+			dirpath = g_build_filename(abs_base_dir, part, NULL);
+			if (dirpath != NULL && g_file_test (dirpath, G_FILE_TEST_IS_DIR))
+			{
+				dir_list = g_slist_prepend(dir_list, path_arr);
+				g_free(dirpath);
+			}
+			else
+			{
+				file_list = g_slist_prepend(file_list, path_arr);
+			}
 		}
 	}
 
 	foreach_slist (elem, file_list)
 	{
 		GtkTreeIter iter;
-		gchar *part, *full;
 		GIcon *icon = NULL;
 
 		path_arr = elem->data;
@@ -177,6 +191,10 @@ static void sidebar_create_branch(gint level, const gchar *abs_base_dir, GSList
 		{
 			gboolean dir_changed;
 
+			part = g_build_filenamev(path_arr);
+			full = g_build_filename(abs_base_dir, part, NULL);
+			g_free(part);
+
 			path_arr = (gchar **) elem->data;
 			dir_changed = g_strcmp0(last_dir_name, path_arr[level]) != 0;
 
@@ -185,7 +203,8 @@ static void sidebar_create_branch(gint level, const gchar *abs_base_dir, GSList
 				gtk_tree_store_insert_with_values(sidebar.file_store, &iter, parent, 0,
 					FILEVIEW_COLUMN_ICON, icon_dir,
 					FILEVIEW_COLUMN_NAME, last_dir_name,
-					FILEVIEW_COLUMN_DATA_ID, DATA_ID_FOLDER,
+					FILEVIEW_COLUMN_DATA_ID, DATA_ID_SUB_DIRECTORY,
+					FILEVIEW_COLUMN_ASSIGNED_DATA_POINTER, g_strdup(full),
 					-1);
 
 				sidebar_create_branch(level+1, abs_base_dir, tmp_list, &iter);
@@ -194,15 +213,22 @@ static void sidebar_create_branch(gint level, const gchar *abs_base_dir, GSList
 				tmp_list = NULL;
 				last_dir_name = path_arr[level];
 			}
+			g_free(full);
 
 			tmp_list = g_slist_prepend(tmp_list, path_arr);
 		}
 
+		part = g_build_filenamev(path_arr);
+		full = g_build_filename(abs_base_dir, part, NULL);
+		g_free(part);
+
 		gtk_tree_store_insert_with_values(sidebar.file_store, &iter, parent, 0,
 			FILEVIEW_COLUMN_ICON, icon_dir,
 			FILEVIEW_COLUMN_NAME, last_dir_name,
-			FILEVIEW_COLUMN_DATA_ID, DATA_ID_FOLDER,
+			FILEVIEW_COLUMN_DATA_ID, DATA_ID_SUB_DIRECTORY,
+			FILEVIEW_COLUMN_ASSIGNED_DATA_POINTER, g_strdup(full),
 			-1);
+			g_free(full);
 
 		sidebar_create_branch(level+1, abs_base_dir, tmp_list, &iter);
 
@@ -221,11 +247,11 @@ static void sidebar_create_branch(gint level, const gchar *abs_base_dir, GSList
 /* Reverse strcmp */
 static int rev_strcmp(const char *str1, const char *str2)
 {
-	return strcmp(str2, str1);
+	return g_strcmp0(str2, str1);
 }
 
 
-/* Insert given directory/folder into the sidebar file tree */
+/* Insert given directory/sub-directory into the sidebar file tree */
 static void sidebar_insert_project_directory(WB_PROJECT *prj, WB_PROJECT_DIR *directory, GtkTreeIter *parent)
 {
 	GSList *lst = NULL;
@@ -241,7 +267,10 @@ static void sidebar_insert_project_directory(WB_PROJECT *prj, WB_PROJECT_DIR *di
 	while (g_hash_table_iter_next(&iter, &key, &value))
 	{
 		gchar *path = get_relative_path(abs_base_dir, key);
-		lst = g_slist_prepend(lst, path);
+		if (path != NULL)
+		{
+			lst = g_slist_prepend(lst, path);
+		}
 	}
 	/* sort in reverse order so we can prepend nodes to the tree store -
 	 * the store behaves as a linked list and prepending is faster */
@@ -595,7 +624,8 @@ static void sidebar_update_workbench(GtkTreeIter *iter, gint *position)
 		gchar text[200];
 
 		count = workbench_get_project_count(wb_globals.opened_wb);
-		length = g_snprintf(text, sizeof(text), _("%s: %u Projects"),
+		length = g_snprintf(text, sizeof(text),
+							g_dngettext(GETTEXT_PACKAGE, "%s: %u Project", "%s: %u Projects", count),
 							workbench_get_name(wb_globals.opened_wb), count);
 		if (length < (gint)(sizeof(text)-1) && workbench_is_modified(wb_globals.opened_wb))
 		{
@@ -756,9 +786,9 @@ static gboolean sidebar_file_view_on_button_release(G_GNUC_UNUSED GtkWidget * wi
 			{
 				popup_context = POPUP_CONTEXT_FILE;
 			}
-			else if (context.folder != NULL)
+			else if (context.subdir != NULL)
 			{
-				popup_context = POPUP_CONTEXT_FOLDER;
+				popup_context = POPUP_CONTEXT_SUB_DIRECTORY;
 			}
 			else if (context.directory != NULL)
 			{
@@ -959,8 +989,11 @@ gboolean sidebar_file_view_get_selected_context(SIDEBAR_CONTEXT *context)
 					case DATA_ID_NO_DIRS:
 						/* Has not got any data. */
 					break;
-					case DATA_ID_FOLDER:
-						context->folder = data;
+					case DATA_ID_SUB_DIRECTORY:
+						if (context->subdir == NULL)
+						{
+							context->subdir = data;
+						}
 					break;
 					case DATA_ID_FILE:
 						context->file = data;
@@ -998,7 +1031,7 @@ static void sidebar_get_filelist_for_iter(GPtrArray *list, GtkTreeIter iter)
 				g_ptr_array_add(list, g_strdup(filename));
 			break;
 			case DATA_ID_DIRECTORY:
-			case DATA_ID_FOLDER:
+			case DATA_ID_SUB_DIRECTORY:
 				if (gtk_tree_model_iter_children(model, &childs, &iter) == TRUE)
 				{
 					sidebar_get_filelist_for_iter(list, childs);
@@ -1012,7 +1045,7 @@ static void sidebar_get_filelist_for_iter(GPtrArray *list, GtkTreeIter iter)
 
 
 /* Get the lkist of files belonging to the current selection for
-   id (id = project, directory, folder) */
+   id (id = project, directory, sub-directory) */
 static GPtrArray *sidebar_get_selected_filelist (guint id)
 {
 	GtkTreeModel *model;
@@ -1056,14 +1089,14 @@ GPtrArray *sidebar_get_selected_directory_filelist (void)
 }
 
 
-/** Get the list of files corresponding to the selected folder.
+/** Get the list of files corresponding to the selected sub-directory.
  * 
  * @return GPtrArray containing file names or NULL.
  *
  **/
-GPtrArray *sidebar_get_selected_folder_filelist (void)
+GPtrArray *sidebar_get_selected_subdir_filelist (void)
 {
-	return sidebar_get_selected_filelist(DATA_ID_FOLDER);
+	return sidebar_get_selected_filelist(DATA_ID_SUB_DIRECTORY);
 }
 
 /** Setup the sidebar.
